function multi_dof(n)
% Function that applies mixed rational approximation
% of the psi_1 function to the equations of motion
% for a multi-degree of freedom mass-spring system.
% Input n is the number of masses.

% Add path for structured computation, plus expint and chebfun (modify as needed):
%
% addpath /home/paola/Documents/Documents/MATLAB/luca_phi/expint/common
% addpath /home/paola/Documents/Documents/MATLAB/chebfun-master/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define psi_1 function:
f=@(z)(z./(exp(z)-1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Choose degrees:
np=3;     % polynomial part of mixed approx (# of terms)
ns=10;    % rational part of mixed approx (# of terms)
k=7;      % degree of diagonal Padé approx


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define test matrix a and compute b=psi_1(a).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Set up the oscillatory system.
%
% Define physical constants:

m=ones(n,1);  % masses
kconst= 0.3*ones(n+1,1); %kconst=rand(n+1,1);  % elastic constants
lambda=0.1*ones(n,1); %lambda=rand(n,1);  % drag constants
% Form matrices:
M=diag(m);
K=diag(kconst(1:n)+kconst(2:n+1))+diag(-kconst(2:n),1)+diag(-kconst(2:n),-1);
C=diag(lambda);
tau=1;  % time interval
a=tau*[zeros(n) eye(n); -M\K -M\C];   % a is already multiplied by tau

% Compute polynomial and mixed approximations.
 [bmix,bpoly]=mixed_approx(a, np, ns);

% Padé approx using EXPINT:
bpade=phipade(a,1,k);
bpadeinv=inv(bpade);

% Padé approx using EXPM:
bpadeexpm=a/(expm(a)-eye(size(a)));

% Choose boundary conditions and compute p vector 
gvec=[0.5*ones(n,1);zeros(n,1)];   % initial condition
pvec=0.5*ones(n,1);   % constant external force
myfun=@(t,x)a*x/tau+[zeros(n,1);pvec];
options = odeset('RelTol',1e-13,'AbsTol',1e-13);
[yy,zz]=ode45(myfun, [0,tau], gvec, options);
hvec=zz(end,:)';  % boundary condition at t = tau

% No structure:
cp_unstr=bmix*(hvec-gvec)-a*gvec/tau;  % computed p vector

% 2x2 block structure:
cp_two = two_mixed_approx(a,hvec-gvec,np,ns)/tau-a*gvec/tau;

disp('*** For unstructured mixed approximation ***')
disp('Norm of first part of p vector (should be zero):')
disp(norm(cp_unstr(1:n)))
disp('Absolute error on p:')
disp(norm(cp_unstr(n+1:2*n)-pvec))
disp('Relative error on p:')
disp(norm(cp_unstr(n+1:2*n)-pvec)/norm(pvec))

disp('*** For 2x2-block-structured mixed approximation ***')
disp('Norm of first part of p vector (should be zero):')
disp(norm(cp_two(1:n)))
disp('Absolute error on p:')
disp(norm(cp_two(n+1:2*n)-pvec))
disp('Relative error on p:')
disp(norm(cp_two(n+1:2*n)-pvec)/norm(pvec))


cpexpint=bpade\(hvec-gvec)/tau-a*gvec/tau;
disp('*** For EXPINT approximation ***')
disp('Norm of first part of p vector (should be zero):')
disp(norm(cpexpint(1:n)))
disp('Absolute error on p:')
disp(norm(cpexpint(n+1:2*n)-pvec))
disp('Relative error on p:')
disp(norm(cpexpint(n+1:2*n)-pvec)/norm(pvec))

cpexpm=bpadeexpm*(hvec-gvec)/tau-a*gvec/tau;
disp('*** For expm approximation ***')
disp('Norm of first part of p vector (should be zero):')
disp(norm(cpexpm(1:n)))
disp('Absolute error on p:')
disp(norm(cpexpm(n+1:2*n)-pvec))
disp('Relative error on p:')
disp(norm(cpexpm(n+1:2*n)-pvec)/norm(pvec))


